/*  udpbench - A RFC865 compliant benchmarking client.
    Copyright (C) 2000 Afghan Hound Software

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA*/

/* udpbench.c - connects to server as many times per second as it can */

#include "main.h"

void sig_alrm(int sig)
{
  got_alarm = 1;
  if (mark_time)
    {
      alarm(mark_time);
    }
}

int main(int argc, char **argv)
{
  char *host;
  int  port,ret,len;
  struct in_addr *host_addr;
  char qotd[ABS_MAX];
  int sock,con;
  struct sockaddr_in their_addr;
  struct sockaddr_in my_addr;
  struct sigaction sig;
  long   last,current,total;
  struct timeval response;
  
  mark_time = 0;
  parse_bench_option(argc,argv,"udp");
  check_bench_options();
  
  if ((mark_time)&&(!(interval_time)))
    {
      sig.sa_handler = sig_alrm;
      sig.sa_flags = SA_RESTART;
      sigemptyset(&sig.sa_mask);
      sigaction(SIGALRM,&sig,NULL);
      alarm(mark_time);
    }
  host=(char *)xmalloc(100*sizeof(char));
  host_addr = (struct in_addr *)xmalloc(sizeof(struct in_addr));

  host=get_host(host);
  port=get_port("udp");
  if ((host_addr=atoaddr(host))==NULL){
    printf("udp_bench: Bad host name, exiting.\n");
    exit(1);
  }
  con   = 0;
  total = 0;
  memset((char *) &their_addr,0,sizeof(their_addr));
  memset((char *) &my_addr,0,sizeof(my_addr));
  their_addr.sin_family   = AF_INET;    
  their_addr.sin_port     = port;   
  their_addr.sin_addr     = *((struct in_addr *)host_addr);
  my_addr.sin_family      = AF_INET;    
  my_addr.sin_port        = htonl(INADDR_ANY);   
  my_addr.sin_addr.s_addr = htons(0);

  while(1)
    {
      (void)gettimeofday(&response,NULL);
      if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) == -1){
	perror("socket");
	exit(1);
      }
      memset((char *) &their_addr,0,sizeof(their_addr));
      memset((char *) &my_addr,0,sizeof(my_addr));
     

      their_addr.sin_family   = AF_INET;    
      their_addr.sin_port     = port;   
      their_addr.sin_addr     = *((struct in_addr *)host_addr);
      my_addr.sin_family      = AF_INET;    
      my_addr.sin_port        = htonl(INADDR_ANY);   
      my_addr.sin_addr.s_addr = htons(0);
  
      if (bind(sock, (struct sockaddr *) &my_addr, sizeof(my_addr)) < 0) {
	perror("bind");
	exit(1);
      }

      if ((ret=sendto(sock,(char *)&ret,1,0,(struct sockaddr *)&their_addr, sizeof(struct sockaddr))) == -1) {
	perror("sendto");
	exit(1);
      }
      
      len = sizeof(my_addr);
      if ((ret=recvfrom(sock,qotd,ABS_MAX,0,(struct sockaddr *)&my_addr,&len)) == -1) {
	perror("recv");
	exit(1);
      }
      
      qotd[ret] = '\0';

      if (log_con)
	{
	  printf("%s",qotd);
	}
      last = response.tv_usec+1000000*response.tv_sec;
      (void)gettimeofday(&response,NULL);
      current = response.tv_usec+1000000*response.tv_sec;
      total += current-last;
      if ((got_alarm)&&(!(interval_time)))
	{
	  fprintf(stderr,"%5d quotes since last mark (%d seconds)",con,mark_time);
	  fprintf(stderr," with average ");
	  fprintf(stderr,"%.2f ms per request.\n",((double)total/((1000*con))));
	  total     = 0;
	  con       = 0;
	  got_alarm = 0;
	}
      con++;
      close(sock);
      if (interval_time)
	{
	  printf("Reponse received: %5d characters in %.2f ms.\n",strlen(qotd),((double)total)/1000);
	  total = 0;
	  sleep(interval_time);
	}
    }
  return 0;
}
